import numpy as np
import pandas as pd
import os


from util import filter_invalid_users, read_usertask, read_userinfo, load_answers
from util import User_groups, Analogy_types, task_ID_list
from util import case_dict, trust_dict, reverse_answer
from scipy import stats

data_folder = "../anonymous_data"
def reserve_user_feedback(reserved_users):
	filename = os.path.join(data_folder, "PostQ_analogy.csv")
	df = pd.read_csv(filename, usecols=['user_id', "answer_1", "answer_2"])
	if reserved_users is not None:
		# filter some invalid users
		df = df.drop(df[~df['user_id'].isin(reserved_users)].index)
	analogy_feedback_list = df.values.tolist()
	familiar_group = set()
	unfamiliar_group = set()
	user_analogy_helpfulness = {}
	for user_id, usefulness, familiarity in analogy_feedback_list:
		user_analogy_helpfulness[user_id] = usefulness
		if familiarity > 3:
			familiar_group.add(user_id)
		else:
			unfamiliar_group.add(user_id)
	return familiar_group, unfamiliar_group, user_analogy_helpfulness

def calc_system_advice(userinfo_dict, answer_dict):
	system_advice_dict = {}
	user_group_dict = {}
	user_analogy_dict = {}
	user_wrong_advice = {}
	for user in userinfo_dict:
		__, task_order_str, reverse_flag_str, user_group, analogy_type = userinfo_dict[user]
		user_group_dict[user] = user_group
		user_analogy_dict[user] = analogy_type
		system_advice_dict[user] = {}
		user_wrong_advice[user] = []
		taks_order_list = task_order_str.split("|")
		reverse_flag_list = reverse_flag_str.split("|")
		for i, task_id in enumerate(taks_order_list):
			correct_answer = answer_dict[task_id]
			reverse_flag = eval(reverse_flag_list[i])
			if reverse_flag:
				user_wrong_advice[user].append(task_id)
				system_advice_dict[user][task_id] = reverse_answer(correct_answer)
			else:
				system_advice_dict[user][task_id] = correct_answer
		assert len(user_wrong_advice[user]) == 3
	return system_advice_dict, user_group_dict, user_analogy_dict

def calc_analogy_domain_usefulness(reserved_users, user_analogy_helpfulness, user_group_dict, user_analogy_dict):
	user_helpfness_dict = {}
	for item in ["train", "weather", "vaccine"]:
		user_helpfness_dict[item] = []
	for user in reserved_users:
		if user_group_dict[user] != "analogy":
			continue
		tp_analogy = user_analogy_dict[user]
		user_helpfness_dict[tp_analogy].append(user_analogy_helpfulness[user])
	statistic, pvalue = stats.kruskal(user_helpfness_dict["train"], user_helpfness_dict["weather"], user_helpfness_dict["vaccine"])
	print("-" * 17)
	# print(statistic)
	print("kruskal test result: H:{:.2f}, p:{:.3f}".format(statistic, pvalue))
	# print("Mean for system: {:.2f}, accuracy: {:.3f}, analogy: {:.3f}".format(np.mean(data_list_1), np.mean(data_list_2), np.mean(data_list_3)))
	# print("SD for system: {:.3f}, accuracy: {:.3f}, analogy: {:.3f}".format(np.std(data_list_1, ddof=1), np.std(data_list_2, ddof=1), np.std(data_list_3, ddof=1)))
	if pvalue >= 0.05:
		print("With pvalue: {:.3f}, the three groups have same population mean with dependent variable {}".format(pvalue, "helpfulness"))
	else:
		print("With pvalue: {:.3f}, the three groups don't have same population mean with dependent variable {}, check it more".format(pvalue, "helpfulness"))
		print("Mean for train: {:.2f}, weather: {:.2f}, vaccine: {:.2f}".format(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"])))
		print("SD for train: {:.2f}, weather: {:.2f}, vaccine: {:.2f}".format(np.std(user_helpfness_dict["train"], ddof=1), np.std(user_helpfness_dict["weather"], ddof=1),\
		 np.std(user_helpfness_dict["vaccine"], ddof=1)))
		# print(np.mean(user_helpfness_dict["train"]), np.mean(user_helpfness_dict["weather"]), np.mean(user_helpfness_dict["vaccine"]))
		statistic, pvalue = stats.mannwhitneyu(user_helpfness_dict["train"], user_helpfness_dict["weather"], alternative='less')
		print("alternative train < weather, pvalue: %.4f"%pvalue, "%.4f"%statistic)
		# print("train: weather")
		statistic, pvalue = stats.mannwhitneyu(user_helpfness_dict["train"], user_helpfness_dict["vaccine"], alternative='less')
		print("alternative train < vaccine, pvalue:  %.4f"%pvalue, "%.4f"%statistic)
		# print("train: vaccine")
		statistic, pvalue = stats.mannwhitneyu(user_helpfness_dict["weather"], user_helpfness_dict["vaccine"], alternative='two-sided')
		print("alternative weather <> vaccine pvalue: %.4f"%pvalue, "%.4f"%statistic)
		# print("weather: vaccine")

def calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict):
	switching_cases = {}
	accuracy_dict = {}
	user_defend = {}
	user_appropriate_reliance = {}
	user_trust_behavior_dict = {}
	user_reliance_list = []
	user_trust_behavior_list = []
	user_correct_list = []
	switching_analogy_cases = {}
	for user in usertask_dict:
		user_group = user_group_dict[user]
		analogy_type = user_analogy_dict[user]
		user_trust_list = []
		tp_correct = 0
		appropriate_reliance = 0
		defend_wrong = 0
		tp_trust = 0
		for task_id in task_ID_list:
			first_choice = usertask_dict[user][(task_id, "base")]
			second_choice = usertask_dict[user][(task_id, user_group)]
			system_advice = system_advice_dict[user][task_id]
			correct_answer = answer_dict[task_id]
			case = case_dict[(first_choice, second_choice, system_advice)]
			trust_pattern = trust_dict[case]
			if trust_pattern in ["weak trust", "strong trust"]:
				tp_trust += 1
			if system_advice != correct_answer:
				# system provide a wrong answer
				if trust_pattern in ["weak distrust"]:
					# participants defend wrong system advice by insisting their first decision
					appropriate_reliance += 1
					defend_wrong += 1
			else:
				# system provide a correct answer
				if trust_pattern in ["weak trust", "strong trust"]:
					# participants adopt correct system advice by switching to system's 
					# or agree with system's advice
					appropriate_reliance += 1
			if second_choice == correct_answer:
				tp_correct += 1
			trust_pattern = trust_dict[case]
			user_trust_list.append(trust_pattern)
		user_appropriate_reliance[user] = appropriate_reliance
		# user_defend[user] = defend_wrong / 3.0
		# switching_cases[user].append(user_trust_list)
		accuracy_dict[user] = tp_correct / 10.0
		user_trust_behavior_dict[user] = tp_trust
	return user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict

if __name__ == "__main__":
	reserved_users = filter_invalid_users(exclude_round1=False)
	usertask_dict, user_attention_check_correct = read_usertask(reserved_users)
	userinfo_dict = read_userinfo(reserved_users)
	familiar_group, unfamiliar_group, user_analogy_helpfulness = reserve_user_feedback(reserved_users)
	answer_dict = load_answers()
	system_advice_dict, user_group_dict, user_analogy_dict = calc_system_advice(userinfo_dict, answer_dict)

	from util import calc_TiA_scale, calc_ATI_scale, TiA_subscales, obtain_analogy_familiarity, calc_numeracy
	user_TiA_scale = calc_TiA_scale()
	user_ATI_scale = calc_ATI_scale()
	user_usefulness_dict, user_familiarity_dict = obtain_analogy_familiarity()
	user_numeracy_dict = calc_numeracy()

	# user_appropriate_reliance, accuracy_dict, user_trust_behavior_dict = calc_appropriate_reliance(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict)

	# calc_switching_familiarity(usertask_dict, system_advice_dict, answer_dict, user_group_dict, user_analogy_dict, familiar_group, unfamiliar_group, user_usefulness_dict)
	# calc_analogy_domain_usefulness(reserved_users, user_analogy_helpfulness, user_group_dict, user_analogy_dict)


	familiarty_user_usefulness_dict = {}
	for user in user_familiarity_dict:
		if user not in reserved_users:
			continue
		if user_group_dict[user] != "analogy":
			continue
		tp_familiarity = user_familiarity_dict[user]
		if tp_familiarity not in familiarty_user_usefulness_dict:
			familiarty_user_usefulness_dict[tp_familiarity] = []
		familiarty_user_usefulness_dict[tp_familiarity].append(user_analogy_helpfulness[user])


	dependent_variables = {
		"helpfuness": familiarty_user_usefulness_dict
	}
	print("-" * 17)
	for index in [1, 2, 3, 4, 5]:
		print("Faimilarity {} has {} participants".format(index, len(familiarty_user_usefulness_dict[index])))
	print("-" * 17)
	for name in dependent_variables:
		statistic, pvalue = stats.kruskal(dependent_variables[name][1], dependent_variables[name][2],\
		dependent_variables[name][3], dependent_variables[name][4],\
		dependent_variables[name][5])
		print("For variable {}".format(name))
		print("Mean for 1: {:.2f}, 2: {:.2f}, 3: {:.2f}, 4: {:.2f}, 5: {:.2f}".format(np.mean(dependent_variables[name][1]), np.mean(dependent_variables[name][2]),\
		 np.mean(dependent_variables[name][3]), np.mean(dependent_variables[name][4]), np.mean(dependent_variables[name][5])))
		print("SD for 1: {:.2f}, 2: {:.2f}, 3: {:.2f}, 4: {:.2f}, 5: {:.2f}".format(np.std(dependent_variables[name][1], ddof=1), np.std(dependent_variables[name][2], ddof=1),\
		 np.std(dependent_variables[name][3], ddof=1), np.std(dependent_variables[name][4], ddof=1), np.std(dependent_variables[name][5], ddof=1)))
		print("statistic: {:.3f}".format(statistic))
		if pvalue >= 0.0125:
			print("With pvalue: {:.3f}, the five groups have same population mean with dependent variable {}".format(pvalue, name))
		else:
			print("With pvalue: {:.3f}, the five groups don't have same population mean with dependent variable {}, check it more".format(pvalue, name))
			for i in [1, 2, 3, 4, 5]:
				for j in [1, 2, 3, 4, 5]:
					if i>= j:
						continue
					statistic, pvalue = stats.mannwhitneyu(dependent_variables[name][i], dependent_variables[name][j], alternative='less')
					# print(i, j, "%.4f"%pvalue)
					print("Alternative {} < {}, with pvalue: {:.4f}".format(i, j, pvalue))
					# print("Mean for {}: {:.2f}; Mean for{}: {:.2f}".format(i, np.mean(dependent_variables[name][i]), j, np.mean(dependent_variables[name][j])))


	calc_analogy_domain_usefulness(reserved_users, user_analogy_helpfulness, user_group_dict, user_analogy_dict)
	# print("-" * 17)

	# print("For dependent variable Usefulness")
	# # print(ancova(data=df, dv='userfulness', covar=['Numeracy_level', 'ATI', 'TiA-Familiarity', 'TiA-Propensity to Trust', 'Analogy_domain_familiarity'], between='analogy_type', effsize='n2'))
	
	# loan_data_variables = {
	# 	"user_id": [],
	# 	"user_group": [],
	# 	"analogy_type": [],
	# 	"Numeracy_level": [],
	# 	"Analogy_domain_familiarity": [],
	# 	"ATI": [],
	# 	"Appropriate_reliance": [],
	# 	"Accuracy": [],
	# 	"Trust_behavior": [],
	# 	"userfulness": []
	# }
	# for subscale in TiA_subscales:
	# 	loan_data_variables["TiA-{}".format(subscale)] = []
	# for user in reserved_users:
	# 	if user_group_dict[user] != "analogy":
	# 		continue
	# 	loan_data_variables["user_id"].append(user)
	# 	# if user_group_dict[user] == "analogy":
	# 	# 	loan_data_variables["user_group"].append(user_group_dict[user] + "-" + user_analogy_dict[user])
	# 	# else:
	# 	loan_data_variables["user_group"].append(user_group_dict[user])
	# 	loan_data_variables["Numeracy_level"].append(user_numeracy_dict[user][0])

	# 	if user_group_dict[user] != "analogy":
	# 		loan_data_variables["analogy_type"].append(None)
	# 		loan_data_variables["Analogy_domain_familiarity"].append(None)
	# 		loan_data_variables["userfulness"].append(None)
	# 	else:
	# 		loan_data_variables["analogy_type"].append(user_analogy_dict[user])
	# 		loan_data_variables["Analogy_domain_familiarity"].append(user_familiarity_dict[user])
	# 		loan_data_variables["userfulness"].append(user_usefulness_dict[user])
	# 	loan_data_variables["ATI"].append(user_ATI_scale[user])
	# 	loan_data_variables["Appropriate_reliance"].append(user_appropriate_reliance[user])
	# 	loan_data_variables["Accuracy"].append(accuracy_dict[user])
	# 	loan_data_variables["Trust_behavior"].append(user_trust_behavior_dict[user])
	# 	for subscale in TiA_subscales:
	# 		loan_data_variables["TiA-{}".format(subscale)].append(user_TiA_scale[user][subscale])
	# for key_ in loan_data_variables:
	# 	print(key_, len(loan_data_variables[key_]))
	# # for key_ in ["ATI", "Numeracy_level", "TiA-Propensity to Trust", "TiA-Familiarity", "Accuracy"]:
	# # 	print(key_)
	# # 	print("mean:{:.2f} std:{:.2f}".format(np.mean(loan_data_variables[key_]), np.std(loan_data_variables[key_], ddof=1)))
	# df = pd.DataFrame(loan_data_variables)
	# from pingouin import ancova
	# print("For dependent variable Usefulness")
	# ### results for table 7
	# print(ancova(data=df, dv='userfulness', covar=['Numeracy_level', 'ATI', 'TiA-Familiarity', 'TiA-Propensity to Trust', 'Analogy_domain_familiarity'], between='analogy_type', effsize='n2'))





